/**
 * Copyright (c) 2008-2011 Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://www.sonatype.com/products/nexus/attributions.
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
package org.sonatype.nexus.plugins.aether;

import org.apache.maven.repository.internal.MavenRepositorySystemSession;
import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.component.annotations.Requirement;
import org.sonatype.aether.RepositoryListener;
import org.sonatype.aether.RepositorySystem;
import org.sonatype.aether.RepositorySystemSession;
import org.sonatype.aether.collection.DependencyGraphTransformer;
import org.sonatype.aether.repository.LocalRepository;
import org.sonatype.aether.util.DefaultRepositorySystemSession;
import org.sonatype.aether.util.graph.transformer.NoopDependencyGraphTransformer;
import org.sonatype.nexus.configuration.application.ApplicationConfiguration;
import org.sonatype.nexus.plugins.aether.workspace.NexusWorkspace;

@Component( role = AetherProvider.class )
public class DefaultAetherProvider
    implements AetherProvider
{
    @Requirement
    private ApplicationConfiguration applicationConfiguration;

    @Requirement
    private RepositorySystem repositorySystem;

    public synchronized RepositorySystem getRepositorySystem()
    {
        return repositorySystem;
    }

    public RepositorySystemSession getDefaultRepositorySystemSession( RepositorySystem repositorySystem )
    {
        MavenRepositorySystemSession session = new MavenRepositorySystemSession();

        // can't aether work _without_ local repo?
        LocalRepository localRepo =
            new LocalRepository( applicationConfiguration.getWorkingDirectory( "aether-local-repository" ) );
        session.setLocalRepositoryManager( repositorySystem.newLocalRepositoryManager( localRepo ) );

        // session.setIgnoreMissingArtifactDescriptor( false );

        // session.setTransferListener( new ConsoleTransferListener( System.out ) );
        // session.setRepositoryListener( new ConsoleRepositoryListener( System.out ) );

        // session.setUpdatePolicy( "" );

        return session;
    }

    public RepositorySystemSession getDefaultRepositorySystemSession( RepositorySystem repositorySystem,
                                                                      NexusWorkspace nexusWorkspace, boolean dirtyTree,
                                                                      RepositoryListener listener )
    {
        DefaultRepositorySystemSession session =
            (DefaultRepositorySystemSession) getDefaultRepositorySystemSession( repositorySystem );

        session.setWorkspaceReader( nexusWorkspace.getWorkspaceReader() );
        session.setOffline( true );
        if ( dirtyTree )
        {
            DependencyGraphTransformer transformer = new NoopDependencyGraphTransformer();
            session.setDependencyGraphTransformer( transformer );
        }
        session.setRepositoryListener( listener );

        return session;
    }
}
